# coding: utf-8

"""本模块的目的是为了增加艺术性和情趣
"""

# 铃木爱理
MM_LINGMUAILI = r"""
XXXXXXXXXXXXXNXXXXNXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKOc,...........,,''';cd0KXXXXXXXXXXXXXXXXXXNXXNNNNNNNNNNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKOo;......................';lxKXXXXXXXXXXXXXXXXNXXNNXNNNNNNNNNNNNNNNNNNNN
NXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKd;.............................':kKXXXXXXXXXXXXXXXXXXNXXNNNNXNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKd,..................................;kKXXXXXXXXXXXXXXXXNNNNNNNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKK0c.....................................':kXXXXXXXXXXXXXXXXXNNNXNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKk;........................................'kKXXXXXXXXXXXXXNXXXXXNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKx'.........................................',kKXXXXXXXXXXXXXXXNNNXNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKK0,.......''''...............................'':0KXXXXXXXXXXXXXXXXXNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKO,.......,''','.....':c......................'',oKXXXXXXXXXXXXXXXXNNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKc.......','',,''''';okx'..................'.'''';xKKXXXXXXXXXXXXXXXNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKK0,.......''.'','''':dOOk;'.''''''''.'....'.'''',',lKKXXXXXXXXXXXXXXNXNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKK0;..........'''...;dO00Ol,'''''''''''''........'',,kKXXXXXXXXXXXXXXNNNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK0c..........''...,dO0000d,'.....................'''cKXXXXXXXXXXXXXXXXNNNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK0O'...............oO00KKKO,........................'c0KXXXXXXXXXXXXXXXXNXNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK0l...............ck00KKKKKc........................'cKKXXXXXXXXXXXXXXXXXNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK0Oc;';...........,dkkkkkkkxo.........................;0KKXXXXXXXXXXXXXXNXNNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK0lc.c:...........lO0KKKKK0Oko'.'';;'',;',;;,,........'OKKKXXXXXXXXXXXXXXXXNNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK0xc:.od,.........;xkdll,,;cxkOd,:doxdoodc::;::cc,.....'OKKKKXXXXXXXXXXXXXXXXNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK00o,..oolc:,.....;x0OxxkollkOkO0OkKKK0O0kd:::xx:lxc,...'OKKKXKKXXXXXXXXXXXXXXNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXKKKKKK000kc...dxdxo;....,x0KKKKKKKKKKKXXKKXXKKKKKK0000OO00Od'..,OKKKKKKXXXXXXXXXXXXXXNNNNNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXXKKKKKK000o,...:xkxd;...,d0KKKXXXXXXXXXXXKKKKKKKKXXXKKKKKKK0k;..;0KKKKKKXXXXXXXXXXXXXXXXNNXNNNNN
XXXXXXXXXXXXXXXXXXXXXXXXKKKKKKK00k;.....,dkxc..'cO00KKKXXXNNXXXXKKKKXXKKKXXXXXXXKKK0x,..c0KKKKKKKXXXXXXXXXXXXXXXNXNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXKKKKKKKK00l'..''...:dl..,lkO00KKXXXXXXXK0000KXXK00KXXXXXXKKK0:..'c0KKKKKKKXXXXXXXXXXXXXXXXXNNNNNN
XXXXXXXXXXXXXXXXXXXXXXXKKKKKKK00Oc'..'.........'lxOO000KXXXXXK0000KXXXK00KXXXXXXKK0l...'l00KKKKKKKXXXXXXXXXXXXXXXXNXNNNN
XXXXXXXXXXXXXXXXXXXXXXKKKKKKK0000..,............:dkO0000KXXXXXK0OkO000OO0KXXXXXXKKx....,d000KKKKKKKXXXXXXXXXXXXXXXXNNNNN
XXXXXXXXXXXXXXXXXXXXXKKKKKKKK00x.................,lxOKK00KKKKKKKXKKKKKK0KKKKKKKKKKO,...';o0000KKKKKKXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXKKKKKKKK00kc.................;oOKKO000OxxOO000KK00000OO0KKKKO;.....;cO000KKKKKKXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXKKKKKKKK00Oo:,......;.........,oKXKOO00KK0OkxxkkOkxdxk0KKKK0x'......;cx000KKKKKKKXXXXXXXXXXXXXXXXXXNX
XXXXXXXXXXXXXXXXXXKKKKKKKK000kc;'........'.......:0KK0kk00KKKKK000000000KKKK0k:....'...,lo000KKKKKKKKXXXXXXXXXXXXXXXXXNX
XXXXXXXXXXXXXXXXKKKKKKKKK000Oocc'..'......'.....'kKKKOodkO0KKKKKKKKKKKKKKK0Ol........,,;doO00KKKKKKKKKXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXKKKKKKKKKK0000xllc'''.,...........oKXK0klloxO0KKKXXXXKKKKKKOl..........,;;ook00KKKKKKKKXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXKKKKKKKKKK00000Oodc;'....'.........;0XKKkoooodxkO0KKKKKKKKK0o'.........,,''':ox0KKKKKKKKKKXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXKKKKKKKKK000000kodc'......,...K0xllOKK0OddddxxxkkkOOOOOOOOxc..........',''..':k00KKKKKKKKKXXXXXXXXXXXXXXXXNX
XXXXXXXXXXXKKKKKKKKKK00000Oxdl;........:00000KKKKKKOddxxkkOOOOOOOOOOOko...........';,'...':dO00KKKKKKKXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXKKKKKKKKK00000OOo:,'........,O0KKKKXXXKKOxxkkkO000000000OOk,............''.'..,:oddxO0KKKKKKXXXXXXXXXXXXXXXXX
XXXXXXXXXXKKKKKKKKKKK000Oko,''........oO00OkkkO0KXXKkkOOO00KKKKKKK00O:'............,,'...;llxOkxxk0KKKKXXXXXXXXXXXXXXXXX
XXXXXXXXXXKKKKKKXXXXXKK0Ox'.''.......l000000KKKK0O00OK0O000KKKKKKKK0l;,.............,,...'dxkOOO0kdkKKKXXXXXXXXXXXXXXXXX
XXXXXXXXXKKXXXXNNNXXXKKO0l...'.......d00KKKK0KKKKKOOk0000K0KKKKKKK0dcc,..............''...okKXKKK00xdKKKXXXXXXXXXXXXXXXX
XXXXXXXKKXXXXNNNNNNNXXK00k...'.......kKKKK0OkkkkO0K0dxO0000KKKKKKKOcdc................'...'oOKNXXXK0kdKKKKXXXXXXXXXXXXXX
XXXXXXKKXXXXNXNXNNNNXXKK00c..........kKKKKK0OOOkkkO0OO00000000KKK0lld,........',...........':kKXXXXKKkkXXXXXXXXXXXXXXXXX
XXXXXKKKXXNNNNNNNNNNXXXKK0O;.........OKKKXX00K0O000KKXKK0OO00KKKK0:kl.........,:......';lll:',cx0KKKKKd0NNNNXXXXXXXXXXXX
XXXXKKKXNNNNXNNNNNXXNXXKKK0x'........kKKXXXK0KK00KKXXXK0OO0KKKKKK0co.........':'.....,cxKKK0lcoxodOKKKOxNNNNNNXXXXXXXXXX
XXXKKKKXXNXXNNNNNNNNNNXKKXK0:.......'O0KXXXXKKKKKKKKKK0OO0KKKKKKKO;'.........;....'..,l0KXXKxOK0OkdOKK0xNNNWNXWXXXXXXXXX
XXKKKKXXNNXXXXNNNNNNNNXK0OOkc......,x0KKKXXXXXKKKKK00O000KKKKKKKK0,..........'...''.;:kKXXKKO0XXK0Od0XOONNNWNNWXXXXXXXXX
XXKKKKXXNNXXNXXNNNXNNKOOOkxl;.....l0KKXXKKXXKK000OkkOO000KKKKKKK0k'....'.........''lol0KXXKXOKXNNNXxkKkNWWNNNWNXXXXXXXXX
XXKKKXXXNNXXNXXNNXXXKKKKkxdo,....dKKXXXXKKKKK000OkkOOO000KKKKK00Ok'..'''......,,.'lkkxKXXXXX0XNNNNN0x0XWWNNNNNNXXXXXXXXX
XKKKKXXXNNXXNXXNNXXNNKKOOkl,,'..:0KXXXXXXXXKK0OkkOOO000KKKKKKKK00k;...':;'....;;;,oOKOXXNNX0XNNNNNXkONWWWNNNNNNNXXXXXXXX
XKKKKXNXNNXNNXXNNNNNX0kkd:;:c...kKXXXXXXXXXKK0kOOO000KKKKKKKKKKKK0o....,;:,....;;:okKOXNNNKKNNNXXXXkKXWWNXNNNNNNNXXXXXXX
"""
